/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once

#include <svx/ActionDescriptionProvider.hxx>
#include <svx/svddrgmt.hxx>
#include <unotools/weakref.hxx>

namespace chart { class DrawViewWrapper; }

namespace chart
{
class ChartModel;

class DragMethod_Base : public SdrDragMethod
{
public:
    DragMethod_Base( DrawViewWrapper& rDrawViewWrapper, OUString aObjectCID
        , const rtl::Reference<::chart::ChartModel>& xChartModel
        , ActionDescriptionProvider::ActionType eActionType = ActionDescriptionProvider::ActionType::Move );
    virtual ~DragMethod_Base() override;

    OUString getUndoDescription() const;

    virtual OUString GetSdrDragComment() const override;
    virtual PointerStyle GetSdrDragPointer() const override;

protected:
    rtl::Reference<::chart::ChartModel> getChartModel() const;

protected:
    DrawViewWrapper&    m_rDrawViewWrapper;
    OUString       m_aObjectCID;
    ActionDescriptionProvider::ActionType m_eActionType;

private:
    unotools::WeakReference<::chart::ChartModel> m_xChartModel;
};

} //  namespace chart

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
